package cn.shuto.plugin.capture;

import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CallbackContext;

import org.apache.cordova.PluginResult;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.mabeijianxi.smallvideorecord2.DeviceUtils;
import com.mabeijianxi.smallvideorecord2.JianXiCamera;
import com.mabeijianxi.smallvideorecord2.MediaRecorderActivity;
import com.mabeijianxi.smallvideorecord2.model.MediaRecorderConfig;

import android.Manifest;
import android.app.Activity;
import android.content.pm.PackageManager;
import android.os.Environment;
import android.util.Log;
import android.content.Intent;
import android.os.Bundle;
import org.apache.cordova.PermissionHelper;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

/**
 * This class echoes a string called from JavaScript.
 */
public class CaptureCordovaPlugin extends CordovaPlugin {
    public static final int REQUEST_CODE = 0x777578;
    private static String LOG_TAG = "CAPTURE_PLUGIN";
    private CallbackContext callbackContext;
    private Map<String,Integer> param = new HashMap<>();

    @Override
    protected void pluginInitialize() {
        // 设置拍摄视频缓存路径
        File dcim = cordova.getActivity().getCacheDir();
        if (DeviceUtils.isZte()) {
            if (dcim.exists()) {
                JianXiCamera.setVideoCachePath(dcim + "/capture/");
            } else {
                JianXiCamera.setVideoCachePath(dcim.getPath().replace("/sdcard/",
                        "/sdcard-ext/")
                        + "/capture/");
            }
        } else {
            JianXiCamera.setVideoCachePath(dcim + "/capture/");
        }
        // 初始化拍摄
        JianXiCamera.initialize(false, null);
    }

    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        this.callbackContext = callbackContext;
        if (action.equals("capture")) {
          int width = args.optInt(0, 360);
          int height = args.optInt(1, 640 );
          int frameRate = args.optInt(2, 30 );
          int bitRate = args.optInt(3,  width * height * 6 );
          int limit = args.optInt(4, 3);
          int duration = args.optInt(5, 10);
          this.param.put("width",width);
          this.param.put("height",height);
          this.param.put("frameRate",frameRate);
          this.param.put("bitRate",bitRate);
          this.param.put("limit",limit);
          this.param.put("duration",duration);
          this.capture();
          return true;
        }
        return false;
    }

    private void capture() throws JSONException {
//        boolean fullScreen = obj.optBoolean("needFull", true);
        MediaRecorderConfig config = new MediaRecorderConfig.Buidler()
                .fullScreen(true)
                .smallVideoWidth(this.param.get("width"))
                .smallVideoHeight(this.param.get("height"))
                .recordTimeMax(this.param.get("duration") * 1000)
                .recordTimeMin(this.param.get("limit") * 1000)
                .maxFrameRate(this.param.get("frameRate"))
                .videoBitrate(this.param.get("bitRate"))
                .captureThumbnailsTime(1)
                .build();
        Intent intentCapture = new Intent(this.cordova.getActivity().getBaseContext(), MediaRecorderActivity.class);
        // intentCapture.putExtra(OVER_ACTIVITY_NAME, overGOActivityName);
        intentCapture.putExtra(MediaRecorderActivity.MEDIA_RECORDER_CONFIG_KEY, config);
        intentCapture.setPackage(this.cordova.getActivity().getApplicationContext().getPackageName());
        this.cordova.startActivityForResult(this, intentCapture, REQUEST_CODE);
    }

    /**
     * Called when the barcode scanner intent completes.
     *
     * @param requestCode The request code originally supplied to startActivityForResult(),
     *                       allowing you to identify who this result came from.
     * @param resultCode  The integer result code returned by the child activity through its setResult().
     * @param intent      An Intent, which can return result data to the caller (various data can be attached to Intent "extras").
     */
    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent intent) {
        if (requestCode == REQUEST_CODE && this.callbackContext != null && intent != null) {
            Bundle bundle = intent.getExtras();
            JSONObject obj = new JSONObject();
            super.onActivityResult(requestCode, resultCode, intent);
            if(resultCode == Activity.RESULT_OK){
                callbackContext.success(bundle.getString(MediaRecorderActivity.VIDEO_URI));
            }else if (resultCode == Activity.RESULT_CANCELED){
                callbackContext.error("");
            }
            else {
              this.callbackContext.error("Unexpected error");
            }
        }
    }



    /**
     * This plugin launches an external Activity when the camera is opened, so we
     * need to implement the save/restore API in case the Activity gets killed
     * by the OS while it's in the background.
     */
    public void onRestoreStateForActivityResult(Bundle state, CallbackContext callbackContext) {
        this.callbackContext = callbackContext;
    }

}
