package com.mabeijianxi.smallvideorecord2;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.media.MediaPlayer;
import android.os.Build;
import android.os.Bundle;
import android.view.MotionEvent;
import android.view.SurfaceView;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.CheckBox;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.mabeijianxi.smallvideorecord2.model.MediaObject;
import com.mabeijianxi.smallvideorecord2.model.MediaRecorderConfig;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

/**
 * 视频录制
 */
public class MediaRecorderActivity extends Activity implements
  MediaRecorderBase.OnErrorListener, OnClickListener, MediaRecorderBase.OnPreparedListener,
  MediaRecorderBase.OnEncodeListener, CircleButtonView.OnLongClickListener,
  MediaPlayer.OnPreparedListener,
  MediaPlayer.OnCompletionListener,
  MediaPlayer.OnErrorListener,
  MediaPlayer.OnInfoListener,
  MediaPlayer.OnSeekCompleteListener,
  MediaPlayer.OnVideoSizeChangedListener{

  protected static final int REQ_PERMISSION_CODE = 0x1000;
  protected int              mGrantedCount       = 0;



  private int RECORD_TIME_MIN = (int) (1.5f * 1000);
  /**
   * 录制最长时间
   */
  private int RECORD_TIME_MAX = 6 * 1000;

  private ImageView mTitleBack;
  /**
   * 前后摄像头切换
   */
  private CheckBox mCameraSwitch;

  /**
   * 闪光灯
   */
  private CheckBox mRecordLed;
  /**
   * 拍摄按钮
   */
  private CircleButtonView mRecordController;

  private TextView redoButton;
  private TextView okButton;

  /**
   * 底部条
   */
  private RelativeLayout mBottomLayout;
  /**
   * 摄像头数据显示画布
   */
  private SurfaceView mSurfaceView;


  /**
   * SDK视频录制对象
   */
  private MediaRecorderBase mMediaRecorder;
  /**
   * 视频信息
   */
  private MediaObject mMediaObject;

  private MediaPlayer mPlayer;

  /**
   * 视屏地址
   */
  public final static String VIDEO_URI = "video_uri";
  /**
   * 本次视频保存的文件夹地址
   */
  public final static String OUTPUT_DIRECTORY = "output_directory";
  /**
   * 视屏截图地址
   */
  public final static String VIDEO_SCREENSHOT = "video_screenshot";
  /**
   * 录制配置key
   */
  public final static String MEDIA_RECORDER_CONFIG_KEY = "media_recorder_config_key";

  private boolean startState;
  private boolean NEED_FULL_SCREEN = false;

  private boolean completed = false;

  @Override
  protected void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON); // 防止锁屏
    if (checkPermission()) {
      onPermissionGranted();
    }
  }

  private void initData() {
    Intent intent = getIntent();
    MediaRecorderConfig mediaRecorderConfig = intent.getParcelableExtra(MEDIA_RECORDER_CONFIG_KEY);
    if (mediaRecorderConfig == null) {
      return;
    }
    NEED_FULL_SCREEN = mediaRecorderConfig.getFullScreen();
    RECORD_TIME_MAX = mediaRecorderConfig.getRecordTimeMax();
    RECORD_TIME_MIN = mediaRecorderConfig.getRecordTimeMin();
    MediaRecorderBase.MAX_FRAME_RATE = mediaRecorderConfig.getMaxFrameRate();
    MediaRecorderBase.NEED_FULL_SCREEN = NEED_FULL_SCREEN;
    MediaRecorderBase.MIN_FRAME_RATE = mediaRecorderConfig.getMinFrameRate();
    MediaRecorderBase.SMALL_VIDEO_HEIGHT = mediaRecorderConfig.getSmallVideoHeight();
    MediaRecorderBase.SMALL_VIDEO_WIDTH = mediaRecorderConfig.getSmallVideoWidth();
    MediaRecorderBase.mVideoBitrate = mediaRecorderConfig.getVideoBitrate();
    MediaRecorderBase.CAPTURE_THUMBNAILS_TIME = mediaRecorderConfig.getCaptureThumbnailsTime();
  }

  private int getId(String idName, String type) {
    return getResources().getIdentifier(idName, type, getPackageName());
  }

  /**
   * 加载视图
   */
  private void loadViews() {
    setContentView(getId("activity_media_recorder", "layout"));
    // ~~~ 绑定控件
    mSurfaceView = (SurfaceView) findViewById(getId("record_preview", "id"));
    mCameraSwitch = (CheckBox) findViewById(getId("record_camera_switcher", "id"));
    mTitleBack = (ImageView) findViewById(getId("title_back", "id"));
    mRecordController = (CircleButtonView) findViewById(getId("record_controller", "id"));
    mBottomLayout = (RelativeLayout) findViewById(getId("bottom_layout", "id"));
    mRecordLed = (CheckBox) findViewById(getId("record_camera_led", "id"));

    redoButton = (TextView) findViewById(getId("record_redo", "id"));
    okButton = (TextView) findViewById(getId("record_ok", "id"));

    mTitleBack.setOnClickListener(this);
    redoButton.setOnClickListener(this);
    okButton.setOnClickListener(this);
    mRecordController.init(this.RECORD_TIME_MIN, this.RECORD_TIME_MAX,this.mMediaObject);
    mRecordController.setOnLongClickListener(this);
    // ~~~ 设置数据

    // 是否支持前置摄像头
    if (MediaRecorderBase.isSupportFrontCamera()) {
      mCameraSwitch.setOnClickListener(this);
    } else {
      mCameraSwitch.setVisibility(View.GONE);
    }
    // 是否支持闪光灯
    if (DeviceUtils.isSupportCameraLedFlash(getPackageManager())) {
      mRecordLed.setOnClickListener(this);
    } else {
      mRecordLed.setVisibility(View.GONE);
    }
  }

  /**
   * 初始化画布
   */
  private void initSurfaceView() {
    if (NEED_FULL_SCREEN) {
      mBottomLayout.setBackgroundColor(0);
      FrameLayout.LayoutParams lp = (FrameLayout.LayoutParams) mSurfaceView
        .getLayoutParams();
      lp.setMargins(0, 0, 0, 0);
      mSurfaceView.setLayoutParams(lp);
    } else {
      final int w = DeviceUtils.getScreenWidth(this);
      ((RelativeLayout.LayoutParams) mBottomLayout.getLayoutParams()).topMargin = (int) (w / (MediaRecorderBase.SMALL_VIDEO_HEIGHT / (MediaRecorderBase.SMALL_VIDEO_WIDTH * 1.0f)));
      int width = w;
      int height = (int) (w * ((MediaRecorderBase.mSupportedPreviewWidth * 1.0f) / MediaRecorderBase.SMALL_VIDEO_HEIGHT));
      FrameLayout.LayoutParams lp = (FrameLayout.LayoutParams) mSurfaceView
        .getLayoutParams();
      lp.width = width;
      lp.height = height;
      mSurfaceView.setLayoutParams(lp);
    }
  }

  /**
   * 初始化拍摄SDK
   */
  private void initMediaRecorder() {
    mMediaRecorder = new MediaRecorderNative();

    mMediaRecorder.setOnErrorListener(this);
    mMediaRecorder.setOnEncodeListener(this);
    mMediaRecorder.setOnPreparedListener(this);

    File f = new File(JianXiCamera.getVideoCachePath());
    if (!FileUtils.checkFile(f)) {
      boolean result = f.mkdirs();
      Log.d("can" + (result ? " " : " *not* ") + "create " + JianXiCamera.getVideoCachePath());
    }
    String key = String.valueOf(System.currentTimeMillis());
    mMediaObject = mMediaRecorder.setOutputDirectory(key,
      JianXiCamera.getVideoCachePath() + key);
    mMediaRecorder.setSurfaceHolder(mSurfaceView.getHolder());
    int screenWidth = DeviceUtils.getScreenWidth(this);
    int screenHegith = DeviceUtils.getScreenHeight(this);
    mMediaRecorder.setScreen(screenWidth, screenHegith);
    mMediaRecorder.prepare();
  }

  @Override
  public void onResume() {
    super.onResume();
    if(checkPermission()){
      if (mMediaRecorder == null) {
        initMediaRecorder();
      } else {
        mRecordLed.setChecked(false);
        mMediaRecorder.prepare();
      }
    }
  }

  /**
   * 开始录制
   */
  private void startRecord() {
    // 删除临时目录

    File file = new File(JianXiCamera.getVideoCachePath());
    if(file.exists() && file.isDirectory()){
      FileUtils.deleteDir(file);
    }
    file.mkdirs();
    if (mMediaRecorder != null) {
      MediaObject.MediaPart part = mMediaRecorder.startRecord();
      if (part == null) {
        return;
      }
    }

    setStartUI();
  }

  private void setStartUI() {
    this.completed = false;
    mCameraSwitch.setEnabled(false);
    mRecordLed.setEnabled(false);
  }

  @Override
  public void onBackPressed() {
    if (mMediaObject != null)
      mMediaObject.delete();
    finish();
  }

  /**
   * 停止录制
   */
  private void stopRecord() {
    if (mMediaRecorder != null) {
      mMediaRecorder.stopRecord();
    }
  }

  private void setPreviewUI(){
    mMediaRecorder.stopPreview();
    mCameraSwitch.setVisibility(View.INVISIBLE);
    mRecordLed.setVisibility(View.INVISIBLE);
    mRecordController.setVisibility(View.INVISIBLE);
    mTitleBack.setVisibility(View.INVISIBLE);

    redoButton.setVisibility(View.VISIBLE);
    okButton.setVisibility(View.VISIBLE);
  }

  private void preview() throws IOException {
    File file = new File(mMediaObject.getOutputTempTranscodingVideoPath());
    if(!file.exists() || file.length() == 0){
      Toast.makeText(this,"录制失败",3000).show();
      return;
    }
    if(mPlayer == null){
      mPlayer = new MediaPlayer();
      mPlayer.setOnCompletionListener(this);
      mPlayer.setOnErrorListener(this);
      mPlayer.setOnInfoListener(this);
      mPlayer.setOnPreparedListener(this);
      mPlayer.setOnSeekCompleteListener(this);
      mPlayer.setOnVideoSizeChangedListener(this);
      mPlayer.setDisplay(mSurfaceView.getHolder());
      mPlayer.reset();
    }
    Log.i("player - file",mMediaObject.getOutputTempTranscodingVideoPath());
    Log.i("player - size",""+file.length());
    mPlayer.setDataSource(mMediaObject.getOutputTempTranscodingVideoPath());
    //使用手机本地视频
    mPlayer.prepareAsync();
  }

  private void setRecordUI(){
    mCameraSwitch.setVisibility(View.VISIBLE);
    mRecordLed.setVisibility(View.VISIBLE);
    mRecordController.setVisibility(View.VISIBLE);
    mTitleBack.setVisibility(View.VISIBLE);
    mSurfaceView.setVisibility(View.VISIBLE);
    redoButton.setVisibility(View.INVISIBLE);
    okButton.setVisibility(View.INVISIBLE);
  }


  private void  redo(){
    mCameraSwitch.setEnabled(true);
    mRecordLed.setEnabled(true);

    MediaObject.MediaPart part = mMediaObject.getCurrentPart();
    if (part != null) {
      if (part.remove) {
        part.remove = false;
        mMediaObject.removePart(part, true);
      } else {
        part.remove = true;
      }
    }
    if(mPlayer != null && mPlayer.isPlaying()){
      mPlayer.reset();
      mMediaRecorder.startPreview();
    }
  }

  private void done(){
    // 复制文件到文件目录
    File file = new File(mMediaObject.getOutputTempTranscodingVideoPath());
    File dir = new File(getDataDir(),"capture");
    if(!dir.exists()  && !dir.isDirectory()){
      dir.mkdirs();
    }
    File out = new File(dir,file.getName());
    boolean success = file.renameTo(out);
    Bundle bundle = new Bundle();
    if(success){
      File temp = new File(JianXiCamera.getVideoCachePath());
      if(temp.exists() && temp.isDirectory()){
        FileUtils.deleteDir(temp);
      }
      temp.mkdirs();
      bundle.putString(MediaRecorderActivity.OUTPUT_DIRECTORY, dir.getAbsolutePath());
      bundle.putString(MediaRecorderActivity.VIDEO_URI, out.getAbsolutePath());
    }else {
      bundle.putString(MediaRecorderActivity.OUTPUT_DIRECTORY, mMediaObject.getOutputDirectory());
      bundle.putString(MediaRecorderActivity.VIDEO_URI, mMediaObject.getOutputTempTranscodingVideoPath());
    }
    Intent resultIntent = new Intent();
    resultIntent.putExtras(bundle);
    this.setResult(RESULT_OK, resultIntent);
    finish();
  }
  @Override
  public void onClick(View v) {
    final int id = v.getId();
    // 处理开启回删后其他点击操作
    if (id != getId("record_delete", "id")) {
      if (mMediaObject != null) {
        MediaObject.MediaPart part = mMediaObject.getCurrentPart();
        if (part != null) {
          if (part.remove) {
            part.remove = false;
          }
        }
      }
    }

    if (id == getId("title_back", "id")) {
      onBackPressed();
    } else if (id == getId("record_camera_switcher", "id")) {// 前后摄像头切换
      if (mRecordLed.isChecked()) {
        if (mMediaRecorder != null) {
          mMediaRecorder.toggleFlashMode();
        }
        mRecordLed.setChecked(false);
      }

      if (mMediaRecorder != null) {
        mMediaRecorder.switchCamera();
      }

      if (mMediaRecorder.isFrontCamera()) {
        mRecordLed.setEnabled(false);
      } else {
        mRecordLed.setEnabled(true);
      }
    } else if (id == getId("record_camera_led", "id")) {// 闪光灯
      // 开启前置摄像头以后不支持开启闪光灯
      if (mMediaRecorder != null) {
        if (mMediaRecorder.isFrontCamera()) {
          return;
        }
      }

      if (mMediaRecorder != null) {
        mMediaRecorder.toggleFlashMode();
      }
    } else if (id == getId("record_redo","id")){
      setRecordUI();
      redo();
    } else if (id == getId("record_ok","id")){
      done();
    }
  }


  @Override
  public void onEncodeStart() {
    Log.d("EncodeStart");
  }

  @Override
  public void onEncodeProgress(int progress) {
    Log.d("onEncodeProgress:"+progress);
  }

  /**
   * 转码完成
   */
  @Override
  public void onEncodeComplete() {
    this.completed = true;
    Log.d("onEncodeComplete");
    try {
      preview();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * 转码失败 检查sdcard是否可用，检查分块是否存在
   */
  @Override
  public void onEncodeError() {
    Log.e("onEncodeError");
    Toast.makeText(this, getString(getId("record_video_transcoding_faild", "string")),
      Toast.LENGTH_SHORT).show();
    finish();
  }

  @Override
  public void onVideoError(int what, int extra) {
    Log.e("onVideoError");
  }

  @Override
  public void onAudioError(int what, String message) {
    Log.e("onAudioError");
  }

  @Override
  public void onPrepared() {
    Log.d("onPrepared");
    initSurfaceView();
  }

  @Override
  protected void onDestroy() {
    super.onDestroy();
    if(mMediaRecorder != null){
      mMediaRecorder.release();
    }
    if(mPlayer != null && mPlayer.isPlaying()){
      mPlayer.stop();
      mPlayer = null;
    }
  }

  private void onPermissionGranted() {
    initData();
    loadViews();
  }
  protected boolean checkPermission() {
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
      List<String> permissions = new ArrayList<>();
      if (PackageManager.PERMISSION_GRANTED != checkSelfPermission(Manifest.permission.CAMERA)) {
        permissions.add(Manifest.permission.CAMERA);
      }
      if (PackageManager.PERMISSION_GRANTED != checkSelfPermission(Manifest.permission.RECORD_AUDIO)) {
        permissions.add(Manifest.permission.RECORD_AUDIO);
      }
      if (PackageManager.PERMISSION_GRANTED != checkSelfPermission(Manifest.permission.READ_EXTERNAL_STORAGE)) {
        permissions.add(Manifest.permission.READ_EXTERNAL_STORAGE);
      }
      if (PackageManager.PERMISSION_GRANTED != checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
        permissions.add(Manifest.permission.WRITE_EXTERNAL_STORAGE);
      }
      if (permissions.size() != 0) {
          requestPermissions(permissions.toArray(new String[0]), REQ_PERMISSION_CODE);
        return false;
      }
    }
    return true;
  }

  @Override
  public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
    super.onRequestPermissionsResult(requestCode, permissions, grantResults);
    switch (requestCode) {
      case REQ_PERMISSION_CODE:
        for (int ret : grantResults) {
          if (PackageManager.PERMISSION_GRANTED == ret) {
            mGrantedCount ++;
          }
        }
        if (mGrantedCount == permissions.length) {
          onPermissionGranted();
        } else {
          Toast.makeText(this, "录制视频缺少必要的权限", Toast.LENGTH_SHORT).show();
        }
        mGrantedCount = 0;
        break;
      default:
        break;
    }
  }

  @Override
  protected void onStop() {
    super.onStop();
    Log.d("onStop");
    if (mMediaRecorder instanceof MediaRecorderNative) {
      ((MediaRecorderNative) mMediaRecorder).activityStop();
    }
  }

  @Override
  public void onLongClick() {
    Log.d("duration:" + mMediaObject.getDuration() + ",curDuration:"+mMediaObject.getCutDuration()+", max:" + RECORD_TIME_MAX);
    if (!startState) {
      startState = true;
      startRecord();
    } else {
      mMediaObject.buildMediaPart(mMediaRecorder.mCameraId);
      setStartUI();
      mMediaRecorder.setRecordState(true);
    }
  }

  @Override
  public void onNoMinRecord(int currentTime) {
    redo();
    startState = false;
    Toast.makeText(MediaRecorderActivity.this, "录制时间太短", 3000).show();
  }

  @Override
  public void onRecordFinishedListener() {
    Log.d("onRecordFinishedListener:拍摄完成");
    startState = false;
    stopRecord();
    setPreviewUI();
  }

  @Override
  public void onCompletion(MediaPlayer mp) {
    Log.d("player onCompletion");
  }

  @Override
  public boolean onError(MediaPlayer mp, int what, int extra) {
    return false;
  }

  @Override
  public boolean onInfo(MediaPlayer mp, int what, int extra) {
    Log.d("player onInfo");
    return false;
  }

  @Override
  public void onPrepared(MediaPlayer mp) {
    Log.d("player onPrepared");
    if(mPlayer !=null) {
      mPlayer.setLooping(true);
      mPlayer.start();
    }
  }

  @Override
  public void onSeekComplete(MediaPlayer mp) {
    Log.d("player onSeekComplete");
  }

  @Override
  public void onVideoSizeChanged(MediaPlayer mp, int width, int height) {
    Log.d("player onVideoSizeChanged");
  }
}
